###
# Copyright 2016-2021 Hewlett Packard Enterprise, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###

# -*- coding: utf-8 -*-
"""Command to clear pending tasks"""
from __future__ import absolute_import

try:
    from rdmc_helper import (
        LOGGER,
        InvalidCommandLineError,
        InvalidCommandLineErrorOPTS,
        NoChangesFoundOrMadeError,
        NoContentsFoundForOperationError,
        ReturnCodes,
    )
except ImportError:
    from ilorest.rdmc_helper import (
        ReturnCodes,
        InvalidCommandLineError,
        InvalidCommandLineErrorOPTS,
        NoContentsFoundForOperationError,
        NoChangesFoundOrMadeError,
        LOGGER,
    )

from .lib.RestHelpers import RestHelpers


class ClearPendingConfigCommand:
    """Command to clear pending config tasks"""

    def __init__(self):
        self.ident = {
            "name": "clearpmmpendingconfig",
            "usage": None,
            "description": "Clear pmm pending config tasks\n" "\texample: clearpmmpendingconfig",
            "summary": "Clear pending config tasks",
            "aliases": [],
            "auxcommands": [],
        }
        self.cmdbase = None
        self.rdmc = None
        self.auxcommands = dict()

    def get_memory_chunk_tasks(self):
        """
        Function to retrieve Memory Chunk Tasks
        :returns: Retrieved Memory Chunk Tasks
        """
        # Retrieving tasks members
        tasks = RestHelpers(rdmcObject=self.rdmc).retrieve_task_members()
        if tasks:
            # Filtering out Memory Chunk Tasks
            memory_chunk_tasks = RestHelpers(rdmcObject=self.rdmc).filter_task_members(tasks)
            if memory_chunk_tasks:
                return memory_chunk_tasks
        self.rdmc.ui.printer("No pending configuration tasks found.\n\n")
        return []

    def delete_tasks(self, memory_chunk_tasks, verbose=False):
        """
        Function to delete pending configuration tasks
        :param memory_chunk_tasks: Pending confguration tasks.
        :type memory_chunk_tasks: list
        :param verbose: Toggles verbose mode, which print task IDs as
                        individual tasks are deleted.
        :type verbose: Boolean
        :returns: None
        """
        for task in memory_chunk_tasks:
            data_id = task.get("@odata.id")
            task_id = task.get("Id")
            resp = RestHelpers(rdmcObject=self.rdmc).delete_resource(data_id)
            if resp:
                if verbose:
                    self.rdmc.ui.printer("Deleted Task #{}".format(task_id) + "\n")
            else:
                raise NoChangesFoundOrMadeError("Error occured while deleting " "task #{}".format(task_id))
        return None

    def run(self, line, help_disp=False):
        """
        Wrapper function for new command main function
        :param line: command line input
        :type line: string.
        """
        if help_disp:
            self.parser.print_help()
            return ReturnCodes.SUCCESS
        LOGGER.info("Clear Pending Configuration: %s", self.ident["name"])
        # pylint: disable=unused-variable
        try:
            (options, args) = self.rdmc.rdmc_parse_arglist(self, line)
        except (InvalidCommandLineErrorOPTS, SystemExit):
            if ("-h" in line) or ("--help" in line):
                return ReturnCodes.SUCCESS
            else:
                raise InvalidCommandLineError("Failed to parse options")
        if args:
            raise InvalidCommandLineError("Chosen flag doesn't expect additional arguments")
        # Raise exception if server is in POST
        if RestHelpers(rdmcObject=self.rdmc).in_post():
            raise NoContentsFoundForOperationError(
                "Unable to retrieve resources - " "server might be in POST or powered off"
            )
        memory_chunk_tasks = self.get_memory_chunk_tasks()
        self.delete_tasks(memory_chunk_tasks, verbose=True)

        return ReturnCodes.SUCCESS

    def definearguments(self, customparser):
        """
        Wrapper function for new command main function
        :param customparser: command line input
        :type customparser: parser.
        """
        if not customparser:
            return

        self.cmdbase.add_login_arguments_group(customparser)



###
# Copyright 2016-2021 Hewlett Packard Enterprise, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###

# -*- coding: utf-8 -*-
""" Single Sign On Command for rdmc """
from argparse import RawDescriptionHelpFormatter

try:
    from rdmc_helper import (
        InvalidCommandLineError,
        InvalidCommandLineErrorOPTS,
        NoContentsFoundForOperationError,
        ReturnCodes,
    )
except ImportError:
    from ilorest.rdmc_helper import (
        InvalidCommandLineError,
        InvalidCommandLineErrorOPTS,
        NoContentsFoundForOperationError,
        ReturnCodes,
    )


class SingleSignOnCommand:
    """Commands Single Sign On actions to the server"""

    def __init__(self):
        self.ident = {
            "name": "singlesignon",
            "usage": None,
            "description": "Add or remove single sign on (SSO) records.\nTo view help on specific "
            "sub-commands run: singlesignon <sub-command> -h\n\nExample: singlesignon "
            "importcert -h\n\n",
            "summary": "Command for all single sign on available actions. ",
            "aliases": ["sso"],
            "auxcommands": [],
        }
        self.cmdbase = None
        self.rdmc = None
        self.auxcommands = dict()

    def run(self, line, help_disp=False):
        """Main SingleSignOnCommand function

        :param line: string of arguments passed in
        :type line: str.
        :param help_disp: display help flag
        :type line: bool.
        """
        if help_disp:
            line.append("-h")
            try:
                (_, _) = self.rdmc.rdmc_parse_arglist(self, line)
            except:
                return ReturnCodes.SUCCESS
            return ReturnCodes.SUCCESS
        try:
            (options, _) = self.rdmc.rdmc_parse_arglist(self, line)
        except (InvalidCommandLineErrorOPTS, SystemExit):
            if ("-h" in line) or ("--help" in line):
                return ReturnCodes.SUCCESS
            else:
                raise InvalidCommandLineErrorOPTS("")

        self.singlesignonvalidation(options)

        actionitem = None
        select = self.rdmc.app.typepath.defs.hpilossotype
        results = self.rdmc.app.select(selector=select)

        try:
            results = results[0]
        except:
            pass

        if results:
            path = results.resp.request.path
        else:
            raise NoContentsFoundForOperationError("%s not found." % select)

        bodydict = results.resp.dict

        if not options.command:
            return ReturnCodes.SUCCESS

        if options.command.lower() == "importdns":
            actionitem = "ImportDNSName"
            body = {"Action": actionitem, "DNSName": options.dnsname}
        elif options.command.lower() == "importcert":
            cert = None
            certtype = None
            actionitem = "ImportCertificate"

            try:
                cert = open(options.cert, "r")

                if cert:
                    certtext = cert.read()
                    cert.close()

                if certtext:
                    certtype = "DirectImportCert"
            except:
                pass

            if not certtype:
                certtype = "ImportCertUri"
                certtext = options.cert

            body = {"Action": actionitem, "CertType": certtype, "CertInput": certtext}

        elif options.command.lower() == "deleterecord":
            if options.record.lower() == "all":
                actionitem = "DeleteAllSSORecords"
                body = {"Action": actionitem}
            else:
                actionitem = "DeleteSSORecordbyNumber"

                try:
                    body = {"Action": actionitem, "RecordNumber": int(options.record)}
                except:
                    raise InvalidCommandLineError("Record to delete must be a number")

        try:
            for item in bodydict["Actions"]:
                if actionitem in item:
                    if self.rdmc.app.typepath.defs.isgen10:
                        actionitem = item.split("#")[-1]
                        body["Action"] = actionitem

                    path = bodydict["Actions"][item]["target"]
                    break
        except:
            pass

        self.rdmc.app.post_handler(path, body)

        self.cmdbase.logout_routine(self, options)
        # Return code
        return ReturnCodes.SUCCESS

    def singlesignonvalidation(self, options):
        """single sign on validation function

        :param options: command line options
        :type options: list.
        """
        self.cmdbase.login_select_validation(self, options)

    def definearguments(self, customparser):
        """Wrapper function for new command main function

        :param customparser: command line input
        :type customparser: parser.
        """
        if not customparser:
            return

        self.cmdbase.add_login_arguments_group(customparser)

        subcommand_parser = customparser.add_subparsers(dest="command")
        save_import_dns_help = "Import a DNS name."
        # importdns sub-parser
        import_dns_parser = subcommand_parser.add_parser(
            "importdns",
            help=save_import_dns_help,
            description=save_import_dns_help + "\n\texample singlesignon importdns dnsname",
            formatter_class=RawDescriptionHelpFormatter,
        )
        import_dns_parser.add_argument("dnsname", help="DNS Name to be imported", metavar="DNSNAME")
        save_import_cert_help = "Import certificate from URI or file."
        # importcert sub-parser
        import_cert_parser = subcommand_parser.add_parser(
            "importcert",
            help=save_import_cert_help,
            description=save_import_cert_help + "\n\texample singlesignon importcert cert.txt",
            formatter_class=RawDescriptionHelpFormatter,
        )
        import_cert_parser.add_argument(
            "cert",
            help="Certificate URI or Certificate File to be imported.",
            metavar="CERTIFICATE",
        )
        # delete sub-parser
        delete_sso_help = "Delete a single or all SSO records."
        delete_sso_parser = subcommand_parser.add_parser(
            "deleterecord",
            help=delete_sso_help,
            description=delete_sso_help + "\nDelete a single record:\nexample deleterecord 2\n"
            "Delete all records:\nexample singlesignon deleterecord all",
            formatter_class=RawDescriptionHelpFormatter,
        )
        delete_sso_parser.add_argument(
            "record",
            help="Record to be deleted (or use keyword 'all' to delete all records)",
            metavar="RECORD",
        )



###
# Copyright 2016-2023 Hewlett Packard Enterprise, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###

# -*- coding: utf-8 -*-
""" Load Command for RDMC """

import json
import os
import shlex
import subprocess
import sys
from datetime import datetime

from six.moves import queue

import redfish.ris
from redfish.ris.rmc_helper import LoadSkipSettingError

try:
    from rdmc_helper import (
        Encryption,
        InvalidCommandLineError,
        InvalidCommandLineErrorOPTS,
        InvalidFileFormattingError,
        InvalidFileInputError,
        InvalidMSCfileInputError,
        MultipleServerConfigError,
        NoChangesFoundOrMadeError,
        ReturnCodes,
    )
except ImportError:
    from ilorest.rdmc_helper import (
        Encryption,
        InvalidCommandLineError,
        InvalidCommandLineErrorOPTS,
        InvalidFileFormattingError,
        InvalidFileInputError,
        InvalidMSCfileInputError,
        MultipleServerConfigError,
        NoChangesFoundOrMadeError,
        ReturnCodes,
    )

try:
    from rdmc_helper import HARDCODEDLIST
except:
    from ilorest.rdmc_helper import HARDCODEDLIST

# default file name
__filename__ = "ilorest.json"


class LoadCommand:
    """Constructor"""

    def __init__(self):
        self.ident = {
            "name": "load",
            "usage": None,
            "description": "Run to load the default configuration"
            " file\n\texample: load\n\n\tLoad configuration file from a "
            "different file\n\tif any property values have changed, the "
            "changes are committed and the user is logged out of the server"
            "\n\n\texample: load -f output.json\n\n\tLoad configurations to "
            "multiple servers\n\texample: load -m mpfilename.txt -f output."
            "json\n\n\tNote: multiple server file format (1 server per new "
            "line)\n\t--url <iLO url/hostname> -u admin -p password\n\t--url"
            " <iLO url/hostname> -u admin -p password\n\t--url <iLO url/"
            "hostname> -u admin -p password",
            "summary": "Loads the server configuration settings from a file.",
            "aliases": [],
            "auxcommands": ["CommitCommand", "SelectCommand", "RawPatchCommand"],
        }
        self.filenames = None
        self.mpfilename = None
        self.queue = queue.Queue()
        self.cmdbase = None
        self.rdmc = None
        self.auxcommands = dict()

    def securebootremovereadonly(self, tmp):
        templist = [
            "SecureBootCurrentBoot",
            "@odata.etag",
            "Id",
            "AutoNeg",
            "FQDN",
            "FullDuplex",
            "HostName",
            "IPv6Addresses",
            "IPv6DefaultGateway",
            "LinkStatus",
            "MaxIPv6StaticAddresses",
            "Name",
            "NameServers",
            "PermanentMACAddress",
            "ConfigurationSettings",
            "InterfaceType",
            "NICSupportsIPv6",
            "DomainName",
            "IPV6",
            "DNSServers",
            "MACAddress",
            "VLAN",
            "SpeedMbps",
            "DateTime",
        ]
        remove_data = self.rdmc.app.removereadonlyprops(tmp, False, True, templist)
        return remove_data

    def run(self, line, help_disp=False):
        """Main load worker function

        :param line: command line input
        :type line: string.
        """
        if help_disp:
            self.parser.print_help()
            return ReturnCodes.SUCCESS
        try:
            (options, _) = self.rdmc.rdmc_parse_arglist(self, line)
        except (InvalidCommandLineErrorOPTS, SystemExit):
            if ("-h" in line) or ("--help" in line):
                return ReturnCodes.SUCCESS
            else:
                raise InvalidCommandLineErrorOPTS("")

        self.loadvalidation(options)
        returnvalue = False

        if options.mpfilename:
            self.rdmc.ui.printer("Loading configuration for multiple servers...\n")
        else:
            self.rdmc.ui.printer("Loading configuration...\n")

        for files in self.filenames:
            if not os.path.isfile(files):
                raise InvalidFileInputError(
                    "File '%s' doesn't exist. Please create file by running save command." % files
                )
            if options.encryption:
                with open(files, "rb") as myfile:
                    data = myfile.read()
                    data = Encryption().decrypt_file(data, options.encryption)
                    loadcontents = json.loads(data)
            else:
                try:
                    with open(files, "r") as myfile:
                        loadcontents = json.load(myfile)
                except:
                    raise InvalidFileFormattingError("Invalid file formatting found in file %s" % files)

            if options.mpfilename:
                mfile = options.mpfilename
                outputdir = None

                if options.outdirectory:
                    outputdir = options.outdirectory

                if self.runmpfunc(mpfile=mfile, lfile=files, outputdir=outputdir):
                    return ReturnCodes.SUCCESS
                else:
                    raise MultipleServerConfigError("One or more servers failed to load given configuration.")

            results = False
            eth_conf = False
            validation_errs = []

            for loadcontent in loadcontents:
                for content, loaddict in loadcontent.items():
                    inputlist = list()

                    if content == "Comments":
                        continue

                    if "EthernetInterface" in content:
                        if options.force_network_config:
                            import platform
                            import tempfile

                            patchpath = "/redfish/v1/Managers/1/EthernetInterfaces/1/"

                            try:
                                payload = loadcontent[content][patchpath]
                            except KeyError:
                                continue

                            payload = loadcontent[content][patchpath]
                            remove_odata = self.securebootremovereadonly(payload)
                            payload.update(remove_odata)
                            if "StaticNameServers" in payload:
                                payload["StaticNameServers"] = [
                                    item for item in payload["StaticNameServers"] if item != "::"
                                ]
                            tempdir = "/tmp" if platform.system() == "Darwin" else tempfile.gettempdir()
                            temp_file = os.path.join(tempdir, "temp_patch.json")
                            out_file = open(temp_file, "w")
                            patch_payload = {patchpath: payload}
                            json.dump(patch_payload, out_file, indent=6)
                            out_file.close()
                            self.auxcommands["rawpatch"].run(temp_file + " --service")
                            os.remove(temp_file)
                            eth_conf = True
                            continue
                        else:
                            self.rdmc.ui.printer(
                                "Skipping network configurations as" " --force_network_config is not included.\n"
                            )
                            continue

                    inputlist.append(content)
                    if options.biospassword:
                        inputlist.extend(["--biospassword", options.biospassword])

                    self.auxcommands["select"].selectfunction(inputlist)
                    if self.rdmc.app.selector.lower() not in content.lower():
                        raise InvalidCommandLineError("Selector not found.\n")

                    try:
                        for _, items in loaddict.items():
                            remove_odata = self.securebootremovereadonly(items)
                            items.update(remove_odata)
                            try:
                                if self.rdmc.app.loadset(
                                    seldict=items,
                                    latestschema=options.latestschema,
                                    uniqueoverride=options.uniqueoverride,
                                ):
                                    results = True
                            except LoadSkipSettingError:
                                returnvalue = True
                                results = True
                            except:
                                raise
                    except redfish.ris.ValidationError as excp:
                        errs = excp.get_errors()
                        validation_errs.append({self.rdmc.app.selector: errs})
                    except:
                        raise

            try:
                if results:
                    self.auxcommands["commit"].commitfunction(options=options)
                    reboot_needed = False
                    for n in inputlist:
                        n = n.lower()
                        if "bios" in n or "boot" in n or "iscsi" in n:
                            reboot_needed = True
                    if reboot_needed:
                        self.rdmc.ui.printer("Reboot is required for settings to take effect.\n")
            except NoChangesFoundOrMadeError as excp:
                if returnvalue:
                    pass
                else:
                    raise excp

            if validation_errs:
                for validation_err in validation_errs:
                    for err_type in validation_err:
                        self.rdmc.ui.error("Validation error(s) in type %s:\n" % err_type)
                        for err in validation_err[err_type]:
                            if isinstance(err, redfish.ris.RegistryValidationError):
                                self.rdmc.ui.error(err.message)
                                try:
                                    if err.reg:
                                        self.rdmc.ui.error(str(err.sel))
                                except:
                                    pass
                raise redfish.ris.ValidationError()

            if not results:
                if eth_conf:
                    continue
                else:
                    self.rdmc.ui.printer("No differences found from current configuration.\n")

        # Return code
        if returnvalue:
            return ReturnCodes.LOAD_SKIP_SETTING_ERROR

        return ReturnCodes.SUCCESS

    def loadvalidation(self, options):
        """Load method validation function

        :param options: command line options
        :type options: list.
        """

        if self.rdmc.opts.latestschema:
            options.latestschema = True

        try:
            self.cmdbase.login_validation(self, options)
        except Exception:
            if options.mpfilename:
                pass
            else:
                raise

        # filename validations and checks
        if options.filename:
            self.filenames = options.filename
        elif self.rdmc.config:
            if self.rdmc.config.defaultloadfilename:
                self.filenames = [self.rdmc.config.defaultloadfilename]

        if not self.filenames:
            self.filenames = [__filename__]

    def verify_file(self, filedata, inputfile):
        """Function used to handle oddly named files and convert to JSON

        :param filedata: input file data
        :type filedata: string.
        :param inputfile: current input file
        :type inputfile: string.
        """
        try:
            tempholder = json.loads(filedata)
            return tempholder
        except:
            raise InvalidFileFormattingError("Invalid file formatting found in file %s" % inputfile)

    def get_current_selector(self, path=None):
        """Returns current selected content minus hard coded list

        :param path: current path
        :type path: string.
        """
        contents = self.rdmc.app.monolith.path[path]

        if not contents:
            contents = list()

        for content in contents:
            for k in list(content.keys()):
                if k.lower() in HARDCODEDLIST or "@odata" in k.lower():
                    del content[k]

        return contents

    def runmpfunc(self, mpfile=None, lfile=None, outputdir=None):
        """Main worker function for multi file command

        :param mpfile: configuration file
        :type mpfile: string.
        :param lfile: custom file name
        :type lfile: string.
        :param outputdir: custom output directory
        :type outputdir: string.
        """
        # self.logoutobj.run("")
        data = self.validatempfile(mpfile=mpfile, lfile=lfile)

        if not data:
            return False

        processes = []
        finalreturncode = True
        outputform = "%Y-%m-%d-%H-%M-%S"

        if outputdir:
            if outputdir.endswith(('"', "'")) and outputdir.startswith(('"', "'")):
                outputdir = outputdir[1:-1]

            if not os.path.isdir(outputdir):
                self.rdmc.ui.error("The give output folder path does not exist.\n")
                raise InvalidCommandLineErrorOPTS("")

            dirpath = outputdir
        else:
            dirpath = os.getcwd()

        dirname = "%s_%s" % (datetime.now().strftime(outputform), "MSClogs")
        createdir = os.path.join(dirpath, dirname)
        os.mkdir(createdir)

        oofile = open(os.path.join(createdir, "CompleteOutputfile.txt"), "w+")
        self.rdmc.ui.printer("Create multiple processes to load configuration " "concurrently to all servers...\n")

        while True:
            if not self.queue.empty():
                line = self.queue.get()
            else:
                break

            finput = "\n" + "Output for " + line[line.index("--url") + 1] + ": \n\n"
            urlvar = line[line.index("--url") + 1]

            if "python" in os.path.basename(sys.executable.lower()):
                # If we are running from source we have to add the python file to the command
                listargforsubprocess = [sys.executable, sys.argv[0]] + line
            else:
                listargforsubprocess = [sys.executable] + line

            if os.name != "nt":
                listargforsubprocess = " ".join(listargforsubprocess)

            urlfilename = urlvar.split("//")[-1]
            logfile = open(os.path.join(createdir, urlfilename + ".txt"), "w+")
            pinput = subprocess.Popen(listargforsubprocess, shell=True, stdout=logfile, stderr=logfile)

            processes.append((pinput, finput, urlfilename, logfile))

        for pinput, finput, urlfilename, logfile in processes:
            pinput.wait()
            returncode = pinput.returncode
            finalreturncode = finalreturncode and not returncode

            logfile.close()
            logfile = open(os.path.join(createdir, urlfilename + ".txt"), "r+")
            oofile.write(finput + str(logfile.read()))
            oofile.write("-x+x-" * 16)
            logfile.close()

            if returncode == 0:
                self.rdmc.ui.printer("Loading Configuration for {} : SUCCESS\n".format(urlfilename))
            else:
                self.rdmc.ui.error("Loading Configuration for {} : FAILED\n".format(urlfilename))
                self.rdmc.ui.error(
                    "ILOREST return code : {}.\nFor more details please check "
                    "{}.txt under {} directory.\n".format(returncode, urlfilename, createdir)
                )

        oofile.close()

        if finalreturncode:
            self.rdmc.ui.printer("All servers have been successfully configured.\n")

        return finalreturncode

    def validatempfile(self, mpfile=None, lfile=None):
        """Validate temporary file

        :param mpfile: configuration file
        :type mpfile: string.
        :param lfile: custom file name
        :type lfile: string.
        """
        self.rdmc.ui.printer("Checking given server information...\n")

        if not mpfile:
            return False

        if not os.path.isfile(mpfile):
            raise InvalidFileInputError(
                "File '%s' doesn't exist, please " "create file by running save command." % mpfile
            )

        try:
            with open(mpfile, "r") as myfile:
                data = list()
                cmdtorun = ["load"]
                cmdargs = ["-f", str(lfile)]
                globalargs = ["-v", "--nocache"]

                while True:
                    line = myfile.readline()

                    if not line:
                        break

                    if line.endswith(os.linesep):
                        line.rstrip(os.linesep)

                    args = shlex.split(line, posix=False)

                    if len(args) < 5:
                        self.rdmc.ui.error("Incomplete data in input file: {}\n".format(line))
                        raise InvalidMSCfileInputError("Please verify the " "contents of the %s file" % mpfile)
                    else:
                        linelist = globalargs + cmdtorun + args + cmdargs
                        line = str(line).replace("\n", "")
                        self.queue.put(linelist)
                        data.append(linelist)
        except Exception as excp:
            raise excp

        if data:
            return data

        return False

    def definearguments(self, customparser):
        """Wrapper function for new command main function

        :param customparser: command line input
        :type customparser: parser.
        """
        if not customparser:
            return

        self.cmdbase.add_login_arguments_group(customparser)

        customparser.add_argument(
            "-f",
            "--filename",
            dest="filename",
            help="Use this flag if you wish to use a different"
            " filename than the default one. The default filename is"
            " %s." % __filename__,
            action="append",
            default=None,
        )
        customparser.add_argument(
            "-m",
            "--multiprocessing",
            dest="mpfilename",
            help="""use the provided filename to obtain data""",
            default=None,
        )
        customparser.add_argument(
            "--outputdirectory",
            dest="outdirectory",
            help="""use the provided directory to output data for multiple server configuration""",
            default=None,
        )
        customparser.add_argument(
            "--latestschema",
            dest="latestschema",
            action="store_true",
            help="Optionally use the latest schema instead of the one "
            "requested by the file. Note: May cause errors in some data "
            "retrieval due to difference in schema versions.",
            default=None,
        )
        customparser.add_argument(
            "--uniqueoverride",
            dest="uniqueoverride",
            action="store_true",
            help="Override the measures stopping the tool from writing " "over items that are system unique.",
            default=False,
        )
        customparser.add_argument(
            "--encryption",
            dest="encryption",
            help="Optionally include this flag to encrypt/decrypt a file " "using the key provided.",
            default=None,
        )
        customparser.add_argument(
            "--reboot",
            dest="reboot",
            help="Use this flag to perform a reboot command function after"
            " completion of operations.  For help with parameters and"
            " descriptions regarding the reboot flag, run help reboot.",
            default=None,
        )
        customparser.add_argument(
            "--force_network_config",
            dest="force_network_config",
            help="Use this flag to force set network configuration."
            "Network settings will be skipped if the flag is not included.",
            action="store_true",
            default=None,
        )



###
# Copyright 2016-2021 Hewlett Packard Enterprise, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###

# -*- coding: utf-8 -*-
""" RawPut Command for rdmc """

import json
import re
import sys
from collections import OrderedDict

try:
    from rdmc_helper import (
        InvalidCommandLineError,
        InvalidCommandLineErrorOPTS,
        InvalidFileFormattingError,
        InvalidFileInputError,
        ReturnCodes,
    )
except ImportError:
    from ilorest.rdmc_helper import (
        InvalidCommandLineError,
        InvalidCommandLineErrorOPTS,
        InvalidFileFormattingError,
        InvalidFileInputError,
        ReturnCodes,
    )


class RawPutCommand:
    """Raw form of the put command"""

    def __init__(self):
        self.ident = {
            "name": "rawput",
            "usage": None,
            "description": "Run to send a post from "
            "the data in the input file.\n\tMultiple PUTs can be performed in sequence by "
            "\n\tadding more path/body key/value pairs.\n"
            "\n\texample: rawput rawput."
            'txt\n\n\tExample input file:\n\t{\n\t    "/redfish/'
            'v1/systems/(system ID)/bios/Settings/":\n\t    {\n\t'
            '\t"Attributes": {\n\t\t'
            '  "BaseConfig": "default"\n\t\t}\n\t    }\n\t}',
            "summary": "Raw form of the PUT command.",
            "aliases": [],
            "auxcommands": [],
        }
        self.cmdbase = None
        self.rdmc = None
        self.auxcommands = dict()

    def run(self, line, help_disp=False):
        """Main raw put worker function

        :param line: command line input
        :type line: string.
        :param help_disp: display help flag
        :type line: bool.
        """
        if help_disp:
            self.parser.print_help()
            return ReturnCodes.SUCCESS
        try:
            (options, args) = self.rdmc.rdmc_parse_arglist(self, line)
        except (InvalidCommandLineErrorOPTS, SystemExit):
            if ("-h" in line) or ("--help" in line):
                return ReturnCodes.SUCCESS
            else:
                raise InvalidCommandLineErrorOPTS("")

        headers = {}
        results = []

        if hasattr(options, "sessionid") and options.sessionid:
            self.sessionvalidation(options)
        else:
            self.putvalidation(options)

        contentsholder = None

        try:
            with open(options.path, "r") as _if:
                contentsholder = json.loads(_if.read(), object_pairs_hook=OrderedDict)
        except IOError:
            raise InvalidFileInputError(
                "File '%s' doesn't exist. " "Please create file by running 'save' command." % options.path
            )
        except ValueError:
            raise InvalidFileFormattingError("Input file '%s' was not " "formatted properly." % options.path)

        if options.headers:
            extraheaders = options.headers.split(",")

            for item in extraheaders:
                header = item.split(":")

                try:
                    headers[header[0]] = header[1]
                except:
                    raise InvalidCommandLineError("Invalid format for --headers option.")

        if "path" in contentsholder and "body" in contentsholder:
            results.append(
                self.rdmc.app.put_handler(
                    contentsholder["path"],
                    contentsholder["body"],
                    headers=headers,
                    silent=options.silent,
                    optionalpassword=options.biospassword,
                    service=options.service,
                )
            )
        elif all([re.match("^/(S+/?)+$", key) for key in contentsholder]):
            for path, body in contentsholder.items():
                results.append(
                    self.rdmc.app.put_handler(
                        path,
                        body,
                        headers=headers,
                        silent=options.silent,
                        optionalpassword=options.biospassword,
                        service=options.service,
                    )
                )
        else:
            raise InvalidFileFormattingError("Input file '%s' was not " "formatted properly." % options.path)

        returnresponse = False

        if options.response or options.getheaders:
            returnresponse = True

        if results and returnresponse:
            for result in results:
                if options.getheaders:
                    sys.stdout.write(json.dumps(dict(result.getheaders())) + "\n")

                if options.response:
                    if isinstance(result.read, bytes):
                        sys.stdout.write(result.read.decode("utf-8"))
                    else:
                        sys.stdout.write(result.read)
                    sys.stdout.write("\n")

        self.cmdbase.logout_routine(self, options)
        # Return code
        return ReturnCodes.SUCCESS

    def putvalidation(self, options):
        """Raw put validation function

        :param options: command line options
        :type options: list.
        """
        self.rdmc.login_select_validation(self, options, skipbuild=True)

    def sessionvalidation(self, options):
        """Raw put session validation function

        :param options: command line options
        :type options: list.
        """

        url = None
        if options.user or options.password or options.url:
            if options.url:
                url = options.url
        else:
            if getattr(self.rdmc.app.redfishinst, "base_url", False):
                url = self.rdmc.app.redfishinst.base_url
        if url and "https://" not in url:
            url = "https://" + url

        return url

    def definearguments(self, customparser):
        """Wrapper function for new command main function

        :param customparser: command line input
        :type customparser: parser.
        """
        if not customparser:
            return

        self.cmdbase.add_login_arguments_group(customparser)

        customparser.add_argument(
            "path",
            help="Path to the JSON file containing the data to be patched.",
        )
        customparser.add_argument(
            "--response",
            dest="response",
            action="store_true",
            help="Use this flag to return the iLO response body.",
            default=False,
        )
        customparser.add_argument(
            "--getheaders",
            dest="getheaders",
            action="store_true",
            help="Use this flag to return the iLO response headers.",
            default=False,
        )
        customparser.add_argument(
            "--headers",
            dest="headers",
            help="Use this flag to add extra headers to the request."
            "\t\t\t\t\t Usage: --headers=HEADER:VALUE,HEADER:VALUE",
            default=None,
        )
        customparser.add_argument(
            "--silent",
            dest="silent",
            action="store_true",
            help="""Use this flag to silence responses""",
            default=False,
        )
        customparser.add_argument(
            "--service",
            dest="service",
            action="store_true",
            help="""Use this flag to enable service mode and increase the function speed""",
            default=False,
        )



###
# Copyright 2017 Hewlett Packard Enterprise, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###

# -*- coding: utf-8 -*-
""" AHS diag Command for rdmc """

import os
from ctypes import POINTER, c_char_p, c_int, c_ubyte, create_string_buffer

import redfish.hpilo.risblobstore2 as risblobstore2

try:
    from rdmc_helper import (
        LOGGER,
        InvalidCommandLineError,
        InvalidCommandLineErrorOPTS,
        ReturnCodes,
    )
except ImportError:
    from ilorest.rdmc_helper import (
        LOGGER,
        InvalidCommandLineError,
        InvalidCommandLineErrorOPTS,
        ReturnCodes,
    )
from redfish.hpilo.rishpilo import HpIlo

if os.name != "nt":
    from _ctypes import dlclose


class AHSdiagCommand:
    """Add Sign/Marker posts into the AHS logs of the server"""

    def __init__(self):
        self.ident = {
            "name": "ahsdiag",
            "usage": None,
            "description": "Adding sign or Marker Post into AHS logs."
            "\n\tahsdiag --WriteSignPost \n\tahsdiag --WriteMarkerPost"
            ' --instance 1 --markervalue 3 --markertext "DIMM Test Start"',
            "summary": "Adding sign or Marker Post into AHS logs.",
            "aliases": ["ahsops"],
            "auxcommands": [
                "LoginCommand",
                "SelectCommand",
                "LogoutCommand",
                "ServerlogsCommand",
            ],
        }

        self.dynamicclass = -1
        self.lib = None
        self.channel = None
        self.cmdbase = None
        self.rdmc = None
        self.auxcommands = dict()

    def run(self, line, help_disp=False):
        """Main ahsdiag function

        :param line: string of arguments passed in
        :type line: str.
        :param help_disp: display help flag
        :type line: bool.
        """
        if help_disp:
            self.parser.print_help()
            return ReturnCodes.SUCCESS
        try:
            (options, _) = self.rdmc.rdmc_parse_arglist(self, line)
        except (InvalidCommandLineErrorOPTS, SystemExit):
            if ("-h" in line) or ("--help" in line):
                return ReturnCodes.SUCCESS
            else:
                raise InvalidCommandLineErrorOPTS("")

        self.ahsdiagvalidation(options)
        self.ahsdiagworkerfunction(options)
        self.cmdbase.logout_routine(self, options)
        return ReturnCodes.SUCCESS

    def ahsdiagworkerfunction(self, options):
        """Main ahsdig worker function

        :param options: command line options
        :type options: list.
        """
        if not options:
            raise InvalidCommandLineErrorOPTS("")

        if self.lib is None and self.channel is None:
            self.resetilo()

        if options.signpost and options.markerpost:
            raise InvalidCommandLineErrorOPTS("")

        if options.signpost:
            if options.inst or options.mval or options.mtext:
                raise InvalidCommandLineError("")
            self.addsignpost(options)
        elif options.markerpost:
            if options.inst and options.mval and options.mtext:
                self.addmarkerpost(options)
            else:
                raise InvalidCommandLineErrorOPTS("")
        else:
            self.rdmc.ui.printer("Choose an operation!\n")
            raise InvalidCommandLineErrorOPTS("")

    def addsignpost(self, options):
        """Function to add signpost

        :param options: command line options
        :type options: list.
        """
        excp = None
        for _ in range(0, 3):  # channel loop for iLO
            self.rdmc.ui.printer("Start writing sign post ... \n")
            try:
                if not self.spregister():
                    raise
                self.rdmc.ui.printer("Successfully registered.\n")
                fail = True

                for _ in range(3):
                    if self.signpostregister():
                        self.rdmc.ui.printer("Signpost was successful!\n")
                        fail = False
                        break
                    else:
                        self.rdmc.ui.printer("Signpost attempt failed!\n")

                if options.postIML and fail:
                    self.addimlpost(options)

                self.unregister()
                return
            except Exception:
                if self.rdmc.opts.verbose:
                    self.rdmc.ui.printer("Retrying with new channel...\n")
                self.resetilo()
        raise excp

    def spregister(self):
        """Function that registers the signpost"""
        self.bb_class_allocate()

        if self.dynamicclass == -1:
            return False

        self.lib.bb_register_wrapper(self.dynamicclass)
        size = self.lib.sizeofbbregister()
        rcode = self.dowriteread(size)

        if rcode == 0:
            self.lib.bb_descriptor_code_wrapper(self.dynamicclass)
            size = self.lib.sizeofbbdescriptorcode()
            rcode = self.dowriteread(size)

        #         if rcode > 0:
        #             return True
        #         else:
        #             return False
        return True

    def bb_class_allocate(self):
        """Function to obtain a handle for further operations"""
        self.lib.bb_class_allocate_wrapper()
        self.lib.getreq.restype = POINTER(c_ubyte)
        ptrreq = self.lib.getreq()

        # sizeofbbchifresp = self.lib.sizeofBBCHIFRESP()
        size = self.lib.sizeofbbclassalloc()
        data = bytearray(ptrreq[:size][:])

        if self.rdmc.opts.debug:
            LOGGER.info("Sending data to chif:{0}\n".format(bytes(data)))

        resp = self.channel.chif_packet_exchange(data)

        resp = create_string_buffer(bytes(resp))
        self.lib.setresp.argtypes = [c_char_p]
        self.lib.setresp(resp)

        if len(resp) != 0:
            self.dynamicclass = self.lib.respmsgrcode()
            self.rdmc.ui.printer("Do Read Return error code:{0}\n".format(self.dynamicclass))
        else:
            self.dynamicclass = 0

    def unregister(self):
        """Function to unregister the handle"""
        self.lib.bb_unregister_wrapper()
        size = self.lib.sizeinunregister()
        rcode = self.dowriteread(size)

        if rcode == 0:
            self.rdmc.ui.printer("Unregistration successful!\n")
        else:
            self.rdmc.ui.printer("Failed to unregister with code:{0}.\n".format(rcode))

    def dowriteread(self, size):
        """Function responsible for communication with the iLO

        :param size: size of the buffer to be read or written
        :type size: int
        """
        ptrreq = self.lib.getreq()
        data = bytearray(ptrreq[:size][:])
        if self.rdmc.opts.debug:
            LOGGER.info("Sending data to chif:{0}\n".format(data))
        try:
            resp = self.channel.chif_packet_exchange(data)
        except Exception:
            self.resetilo()
            return -1

        self.lib.setresp.argtypes = [c_char_p]
        resp = create_string_buffer(bytes(resp))
        self.lib.setresp(resp)
        rcode = self.lib.respmsgrcode()

        if rcode == 0:
            self.lib.updatehandle()

        return rcode

    def signpostregister(self):
        """Worker function of signpost register"""
        self.lib.bb_code_set_wrapper()
        size = self.lib.sizeinbbcodeset()
        returnval = self.dowriteread(size)

        if returnval != 0:
            self.printsignpostregister(returnval)
            return False

        self.lib.bb_log_wrapper()
        size = self.lib.sizeinbblog()
        returnval = self.dowriteread(size)

        if returnval != 0:
            self.printsignpostregister(returnval)
            return False

        self.lib.bb_log_static_wrapper()
        size = self.lib.sizeinbblog()
        returnval = self.dowriteread(size)

        if returnval != 0:
            self.printsignpostregister(returnval)
            return False

        return True

    def printsignpostregister(self, returnval):
        """Commonly used output statement.

        :param returnval: return code for signpost
        :type returnval: int
        """
        self.rdmc.ui.printer("return signpost register failed={0}\n".format(returnval))

    def addmarkerpost(self, options):
        """Function to add marker post

        :param options: command line options
        :type options: list.
        """
        excp = None
        for _ in range(0, 3):  # channel loop for iLO
            self.rdmc.ui.printer("Start writing marker post ... \n")

            try:
                for i in range(3):
                    if self.mpregister():
                        break

                    self.rdmc.ui.printer("attempting to register Marker Post..." "failed {0} times\n".format(i + 1))

                    self.resetilo()

                self.rdmc.ui.printer("Successfully registered.\n")
                fail = True

                for _ in range(3):
                    if self.markerpostregister(options):
                        self.rdmc.ui.printer("Marker Post was successful!\n")
                        fail = False
                        break
                    else:
                        self.rdmc.ui.printer("Marker Post attempt failed!\n")

                if options.postIML and fail:
                    self.addimlpost(options)

                self.unregister()
                return
            except Exception:
                if self.rdmc.opts.verbose:
                    self.rdmc.ui.printer("Retrying with new channel...\n")
                self.resetilo()
        raise excp

    def mpregister(self):
        """Function that registers the marker post"""
        self.bb_class_allocate()
        self.rdmc.ui.printer("return code(dynamic class)={0}.\n".format(self.dynamicclass))

        if self.dynamicclass == -1:
            return False

        self.lib.bb_register_mwrapper(self.dynamicclass)
        size = self.lib.sizeofbbregister()
        rcode = self.dowriteread(size)

        if rcode == 0:
            self.lib.bb_descriptor_code_mwrapper(self.dynamicclass)
            size = self.lib.sizeofbbdescriptorcode()
            rcode = self.dowriteread(size)

            if rcode == -1:
                return False
            self.lib.bb_descriptor_field_mwrapper(self.dynamicclass)
            size = self.lib.sizeofbbdescriptorcode()
            rcode = self.dowriteread(size)
            if rcode == -1:
                return False

        return True

    def markerpostregister(self, options):
        """Worker function of marker post register

        :param options: command line options
        :type options: list.
        """
        self.lib.bb_code_set_mwrapper()
        size = self.lib.sizeinbbcodeset()
        returnval = self.dowriteread(size)

        if returnval != 0:
            return False

        mval = int(options.mval)
        minst = int(options.inst)
        mtext = create_string_buffer(bytes(options.mtext, "utf-8"))

        self.lib.markerpost_wrapper.argtypes = [c_int, c_int, c_char_p]
        self.lib.markerpost_wrapper(mval, minst, mtext)

        size = self.lib.sizeinbblogm()
        returnval = self.dowriteread(size)

        if returnval != 0:
            return False

        return True

    def resetilo(self):
        """Function to reset iLO"""
        if self.channel:
            self.channel.close()
        self.loadlib()
        self.channel = HpIlo(dll=self.lib)
        self.lib.ChifDisableSecurity()

    def loadlib(self):
        """Function to load the so library"""
        self.closedll()
        try:
            if os.name == "nt":
                self.rdmc.ui.printer("Operation can be performed only on Unix based systems!\n")
                raise InvalidCommandLineErrorOPTS("")
            else:
                self.lib = risblobstore2.BlobStore2.gethprestchifhandle()
        except Exception as excp:
            raise InvalidCommandLineErrorOPTS(excp)

    def closedll(self):
        """Deallocate dll handle."""
        try:
            dlclose(self.lib)
        except Exception:
            pass

    def addimlpost(self, options):
        """Adding maintenance post from serverlogs.

        :param options: command line options
        :type options: list.
        """
        if options.signpost:
            imltext = "Signpost Writing Failed"
        elif options.markerpost:
            imltext = "Markerpost Writing Failed"

        options.service = "IML"
        options.clearlog = None
        options.mainmes = imltext

        path = self.serverlogsobj.returnimlpath()
        self.serverlogsobj.addmaintenancelogentry(options, path=path)

    def ahsdiagvalidation(self, options):
        """ahsdiag method validation function"""
        client = None

        try:
            self.cmdbase.login_select_validation(self, options)
            client = self.rdmc.app.current_client
        except Exception:
            if client:
                if not client.base_url == "blobstore://.":
                    raise InvalidCommandLineError("ahsdiag command " "available in local mode only.\n")

            if self.rdmc.config.url:
                raise InvalidCommandLineError("ahsdiag command " "available in local mode only.\n")

            if not client:
                self.lobobj.run("")

    def definearguments(self, customparser):
        """Defines the required arguments for ahsdiag command.

        :param customparser: command line input
        :type customparser: parser.
        """
        if not customparser:
            return

        self.cmdbase.add_login_arguments_group(customparser)

        customparser.add_argument(
            "--selectlog",
            dest="service",
            help="""Read log from the given log service. Options: IML, """ """IEL or AHS.""",
            default=None,
        )
        customparser.add_argument(
            "--clearlog",
            "-c",
            dest="clearlog",
            action="store_true",
            help="""Clears the logs for a the selected option.""",
            default=None,
        )
        customparser.add_argument(
            "--maintenancemessage",
            "-m",
            dest="mainmes",
            help="""Maintenance message to be inserted into the log. """ """(IML LOGS ONLY FEATURE)""",
            default=None,
        )
        customparser.add_argument(
            "--customiseAHS",
            dest="customiseAHS",
            help="""Allows customised AHS log data to be downloaded.""",
            default=None,
        )
        customparser.add_argument(
            "-s",
            "--WriteSignPost",
            dest="signpost",
            action="store_true",
            help="""Writes a sign post into the AHS log.""",
            default=False,
        )
        customparser.add_argument(
            "-r",
            "--WriteMarkerPost",
            dest="markerpost",
            action="store_true",
            help="""Writes a marker post into the AHS log.""",
            default=False,
        )
        customparser.add_argument(
            "-i",
            "--instance",
            dest="inst",
            help="""Argument required by marker post.""",
            default=None,
        )
        customparser.add_argument(
            "-l",
            "--markervalue",
            dest="mval",
            help="""Argument required by marker post.""",
            default=None,
        )
        customparser.add_argument(
            "-t",
            "--markertext",
            dest="mtext",
            help="""Argument required by marker post.""",
            default=None,
        )
        customparser.add_argument(
            "-w",
            "--WriteIMLPost",
            dest="postIML",
            action="store_true",
            help="""Writes an IML entry if failure occurs.""",
            default=False,
        )



###
# Copyright 2016-2021 Hewlett Packard Enterprise, Inc. All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#  http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
###

# -*- coding: utf-8 -*-
"""Command to display information about Persistent Memory modules"""
from __future__ import absolute_import, division

import re
from argparse import Action
from enum import Enum

try:
    from rdmc_helper import (
        LOGGER,
        InvalidCommandLineError,
        InvalidCommandLineErrorOPTS,
        NoContentsFoundForOperationError,
        ReturnCodes,
    )
except ImportError:
    from ilorest.rdmc_helper import (
        ReturnCodes,
        InvalidCommandLineError,
        InvalidCommandLineErrorOPTS,
        NoContentsFoundForOperationError,
        LOGGER,
    )

from .lib.DisplayHelpers import DisplayHelpers, OutputFormats
from .lib.Mapper import Mapper
from .lib.MapperRenderers import MappingTable
from .lib.PmemHelpers import PmemHelpers
from .lib.RestHelpers import RestHelpers


class _Parse_Options_List(Action):
    def __init__(self, option_strings, dest, nargs, **kwargs):
        super(_Parse_Options_List, self).__init__(option_strings, dest, nargs, **kwargs)

    def __call__(self, parser, namespace, values, option_strings):
        """
        Callback to parse a comma-separated list into an array.
        Then store the array in the option's destination
        """
        try:
            setattr(namespace, self.dest, next(iter(values)).split(","))
        except:
            raise InvalidCommandLineError("Values in a list must be separated by a comma.")


class DefaultAttributes(Enum):
    """
    Enum class containing default display attributes for various flags
    """

    device = ["Location", "Capacity", "Status", "DIMMStatus", "Life", "FWVersion"]
    config = ["Location", "VolatileSize", "PmemSize", "PmemInterleaved"]
    summary = ["TotalCapacity", "TotalVolatileSize", "TotalPmemSize"]
    logical = ["PmemSize", "DimmIds"]


def get_default_attributes(flag):
    """
    This method returns default attributes to be displayed
    :param flag: name of flag
    :type flag: string
    :return: list of default attributes for the passed flag
    """
    if flag == "device":
        return list(DefaultAttributes.device.value)
    if flag == "config":
        return list(DefaultAttributes.config.value)
    if flag == "logical":
        return list(DefaultAttributes.logical.value)
    if flag == "summary":
        return list(DefaultAttributes.summary.value)
    return []


class ShowPmemCommand:
    """Command to display information about Persistent Memory modules"""

    def __init__(self):
        self.ident = {
            "name": "showpmm",
            "usage": None,
            "description": "Display information about " "Persistent Memory modules \n\texample: showpmm --device",
            "summary": "Display information about Persistent Memory modules.",
            "aliases": [],
            "auxcommands": [],
        }
        self.cmdbase = None
        self.rdmc = None
        self.auxcommands = dict()
        self._display_helpers = DisplayHelpers()
        self._mapper = Mapper()
        self._pmem_helpers = PmemHelpers()

    def show_pmem_modules(self, options):
        """
        Command to display information about Persistent Memory modules
        :param options: command options
        :type options: options.
        """
        # Retrieving memory collection resources
        if options.logical or options.config:
            memory, domain_members, all_chunks = RestHelpers(rdmcObject=self.rdmc).retrieve_mem_and_mem_domains()
            if not domain_members:
                raise NoContentsFoundForOperationError("Failed to retrieve Memory " "Domain resources")
        else:
            memory = RestHelpers(rdmcObject=self.rdmc).retrieve_memory_resources()

        if memory:
            memory_members = memory.get("Members")
        else:
            raise NoContentsFoundForOperationError("Failed to retrieve Memory Resources")

        # Get dimm ids of all memory objects
        member_dimm_ids = set()
        for member in memory_members:
            member_dimm_ids.add(member.get("DeviceLocator"))

        # Filtering Persistent Memory members
        pmem_members, pmem_dimm_ids = self._pmem_helpers.get_pmem_members(memory_members)
        if not pmem_members:
            raise NoContentsFoundForOperationError("No Persistent Memory Modules found")

        parsed_dimm_ids = list()
        if options.dimm:
            parsed_dimm_ids = self._pmem_helpers.parse_dimm_id(options.dimm)

        for dimm_id in parsed_dimm_ids:
            if dimm_id not in member_dimm_ids:
                raise InvalidCommandLineError("One or more of the specified " "DIMM ID(s) are invalid")
            elif dimm_id not in pmem_dimm_ids:
                raise InvalidCommandLineError(
                    "One or more of the specified DIMM ID(s) " "are not Persistent Memory Modules"
                )

        # Creating a list of persistent memory members according to specified DIMM Ids
        selected_pmem_members = list()
        if parsed_dimm_ids:
            for pmem_member in pmem_members:
                if pmem_member.get("DeviceLocator") in parsed_dimm_ids:
                    selected_pmem_members.append(pmem_member)
        else:
            selected_pmem_members = pmem_members

        # Call 'show_pmem_module_device()' when either the user specifies '--device' flag
        # or specifies no flag at all
        if (
            not options.device and not options.config and not options.logical and not options.summary
        ) or options.device:
            self.show_pmem_module_device(selected_pmem_members, options)

        elif options.config:
            self.show_pmem_module_configuration(selected_pmem_members, all_chunks, options)

        elif options.summary:
            self.show_pmem_module_summary(selected_pmem_members, options)

        elif options.logical:
            if not all_chunks:
                self.rdmc.ui.printer("No Persistent Memory regions found\n\n")
                return
            self.show_persistent_interleave_sets(selected_pmem_members, all_chunks, options)

    def generate_display_output(self, members, options, flag, mapping_table, **resources):
        """
        :param members: list of members returned as a result of GET request
        :type members: list of dictionaries
        :param options: command options
        :type options: options
        :param mapping_table: Mapping table to be used to extract attributes
        :param flag: name of flag
        :type flag: string
        :return: filtered list of dictionaries to be sent to Display Helpers for output
        """
        display_output_list = list()
        # use default attributes for now
        display_attributes = get_default_attributes(flag)

        for member in members:
            temp_dict = self._mapper.get_multiple_attributes(
                member, display_attributes, mapping_table, output_as_json=options.json, **resources
            )
            display_output_list.append(temp_dict)
        return display_output_list

    def show_pmem_module_device(self, selected_pmem_members, options):
        """
        Command to display information about DIMMs when the
        '--device' | '-D' flag is specified
        :param selected_pmem_members: pmem members to be displayed
        :type selected_pmem_members: list
        :param options: command options
        :type options: options
        """
        # generating the data to be printed
        display_output = self.generate_display_output(
            selected_pmem_members, options, "device", MappingTable.device.value
        )
        # Displaying output based on --json flag
        if options.json:
            self._display_helpers.display_data(display_output, OutputFormats.json)
        else:
            self._display_helpers.display_data(display_output, OutputFormats.table)

    def show_pmem_module_configuration(self, selected_pmem_members, all_chunks, options=None):
        """
        Command to display information about DIMMs when the
        '--pmmconfig' | '-C' flag is specified
        :param selected_pmem_members: pmem members to be displayed
        :type selected_pmem_members: list
        :param all_chunks: list of memory chunks
        :type all_chunks: list
        :param options: command options
        :type options: options
        """
        # generating the data to be printed
        display_output = self.generate_display_output(
            selected_pmem_members,
            options,
            "config",
            MappingTable.config.value,
            chunks=all_chunks,
        )
        # Displaying output based on --json flag
        if options.json:
            self._display_helpers.display_data(display_output, OutputFormats.json)
        else:
            self._display_helpers.display_data(display_output, OutputFormats.table)

    def show_persistent_interleave_sets(self, selected_pmem_members, all_chunks, options=None):
        """
        Command to display information about the Persistent interleave
        regions among the Persistent Memory Modules when the '--logical' | '-L' flag is
        specified
        :param selected_pmem_members: list of memory members retrieved via GET request
        :type selected_pmem_members: list
        :param all_chunks: list of memory chunks
        :type all_chunks: list
        :param options: command options
        :type options: options
        """
        # generating the data to be printed
        display_output = self.generate_display_output(
            all_chunks,
            options,
            "logical",
            MappingTable.logical.value,
            memory=selected_pmem_members,
        )
        # Displaying output based on --json flag
        if options.json:
            self._display_helpers.display_data(display_output, OutputFormats.json)
        else:
            self.rdmc.ui.printer("\nInterleave Persistent Memory regions\n")
            self._display_helpers.display_data(display_output, OutputFormats.table)

    def show_pmem_module_summary(self, selected_pmem_members, options=None):
        """
        Command to display a summary of the current Persistent Memory
        configuration when the '--summary' | '-M' flag is specified
        :param selected_pmem_members: list of memory members retrieved via GET request
        :type selected_pmem_members: list
        :param options: command options
        :type options: options
        """
        # getting default attributes for --summary flag
        attribute_list = get_default_attributes("summary")
        # generating the data to be printed
        display_output = self._mapper.get_multiple_attributes(
            selected_pmem_members,
            attribute_list,
            MappingTable.summary.value,
            options.json,
        )
        # Displaying output based on --json flag
        if options.json:
            self._display_helpers.display_data(display_output, OutputFormats.json)
        else:
            self._display_helpers.print_properties([display_output])

    def run(self, line, help_disp=False):
        """
        Wrapper function for new command main function
        :param line: command line input
        :type line: string.
        :param help_disp: display help flag
        :type line: bool.
        """
        if help_disp:
            self.parser.print_help()
            return ReturnCodes.SUCCESS
        LOGGER.info("PMM: %s", self.ident["name"])
        try:
            (options, args) = self.rdmc.rdmc_parse_arglist(self, line)
        except (InvalidCommandLineErrorOPTS, SystemExit):
            if ("-h" in line) or ("--help" in line):
                return ReturnCodes.SUCCESS
            else:
                raise InvalidCommandLineError("Failed to parse options")

        self.rdmc.login_select_validation(self, options)

        if args:
            self.validate_args(options)
        self.validate_show_pmem_options(options)
        # Raise exception if server is in POST
        if RestHelpers(rdmcObject=self.rdmc).in_post():
            raise NoContentsFoundForOperationError(
                "Unable to retrieve resources - " "server might be in POST or powered off"
            )
        self.show_pmem_modules(options)

        return ReturnCodes.SUCCESS

    @staticmethod
    def validate_args(options):
        """
        Produces relevant error messages when unwanted extra arguments are specified with flags
        """
        some_flag = options.device or options.config or options.logical or options.summary
        if options.logical or options.summary:
            raise InvalidCommandLineError("Chosen flag doesn't expect additional arguments")
        elif (options.device or options.config or not some_flag) and not options.dimm:
            raise InvalidCommandLineError("Use the '--dimm | -I' flag to filter by DIMM IDs")
        elif (options.device or options.config or not some_flag) and options.dimm:
            raise InvalidCommandLineError("Values in a list must be comma-separated " "(no spaces)")

    @staticmethod
    def validate_show_pmem_options(options):
        """
        Validate whether options specified by user are valid
        :param options: options specified by the user with the 'showpmm' command
        :type options: instance of OptionParser class
        """
        # Usage/Error strings
        usage_multiple_flags = (
            "Only one of '--device | -D', '--pmmconfig | -C', " "'--logical | -L' or '--summary | -M' may be specified"
        )
        # usage_all_display = "Only one of '--all | -a' or '--display | -d' may be specified\n"
        usage_dimm_flag = (
            "'--dimm | -I' can only be specified  with either the "
            "'--device | -D' or '--pmmconfig | -C' flag\n"
            " or without any flag"
        )
        error_dimm_format = "DIMM IDs should be of the form 'ProcessorNumber@SlotNumber'"
        error_dimm_range = "One or more of the specified DIMM ID(s) are invalid"

        views = [options.device, options.config, options.logical, options.summary]

        if (options.logical or options.summary) and options.dimm:
            raise InvalidCommandLineError(usage_dimm_flag)

        if views.count(False) < 3:
            raise InvalidCommandLineError(usage_multiple_flags)

        if options.dimm:
            for dimm_id in options.dimm:
                # Regex match test for expressions of type 'num1@num2'
                if not re.match(r"^\d+\@\d+$", dimm_id):
                    raise InvalidCommandLineError(error_dimm_format)
                # Accepts expressions from '1@1' to '9@19'
                if not re.match(r"^[1-9]\@([1-9]|[1]\d)$", dimm_id):
                    raise InvalidCommandLineError(error_dimm_range)

    def definearguments(self, customparser):
        """
        Wrapper function for new command main function
        :param customparser: command line input
        :type customparser: parser.
        """
        if not customparser:
            return

        self.cmdbase.add_login_arguments_group(customparser)

        customparser.add_argument(
            "-j",
            "--json",
            action="store_true",
            dest="json",
            help="Optionally include this flag to change the output to JSON format.",
            default=False,
        )

        customparser.add_argument(
            "-D",
            "--device",
            action="store_true",
            default=False,
            dest="device",
            help="Show information about the physical persistent memory modules."
            " Default view shows information about all PMMs."
            " To filter DIMMs, use this flag in conjunction with"
            " the --dimm flag.",
        )

        customparser.add_argument(
            "-C",
            "--pmmconfig",
            action="store_true",
            default=False,
            dest="config",
            help="Show the current configuration of the persistent memory modules."
            " Default view shows information about all PMMs."
            " To filter DIMMs, use this flag in conjunction with"
            " the --dimm flag.",
        )

        customparser.add_argument(
            "-L",
            "--logical",
            action="store_true",
            default=False,
            dest="logical",
            help="Show the Persistent Memory Regions.",
        )

        customparser.add_argument(
            "-M",
            "--summary",
            action="store_true",
            default=False,
            dest="summary",
            help="Show the summary of the persistent memory resources.",
        )

        customparser.add_argument(
            "-I",
            "--dimm",
            type=str,
            action=_Parse_Options_List,
            metavar="IDLIST",
            nargs=1,
            dest="dimm",
            help=" To view specific devices, supply a comma-separated list"
            " of DIMM IDs in the format P@S (without spaces),"
            " where P=processor and S=slot. Example: '1@1,1@12'",
        )
